# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum, EnumMeta
from six import with_metaclass

class _CaseInsensitiveEnumMeta(EnumMeta):
    def __getitem__(self, name):
        return super().__getitem__(name.upper())

    def __getattr__(cls, name):
        """Return the enum member matching `name`
        We use __getattr__ instead of descriptors or inserting into the enum
        class' __dict__ in order to support `name` and `value` being both
        properties for enum members (which live in the class' __dict__) and
        enum members themselves.
        """
        try:
            return cls._member_map_[name.upper()]
        except KeyError:
            raise AttributeError(name)


class Access(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The network traffic is allowed or denied.
    """

    ALLOW = "allow"
    DENY = "deny"

class ClusterState(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The current state of the cluster.
    
    
    * WaitingForNodes - Indicates that the cluster resource is created and the resource provider is
    waiting for Service Fabric VM extension to boot up and report to it.
    * Deploying - Indicates that the Service Fabric runtime is being installed on the VMs. Cluster
    resource will be in this state until the cluster boots up and system services are up.
    * BaselineUpgrade - Indicates that the cluster is upgrading to establishes the cluster version.
    This upgrade is automatically initiated when the cluster boots up for the first time.
    * Upgrading - Indicates that the cluster is being upgraded with the user provided
    configuration.
    * UpgradeFailed - Indicates that the last upgrade for the cluster has failed.
    * Ready - Indicates that the cluster is in a stable state.
    """

    WAITING_FOR_NODES = "WaitingForNodes"
    DEPLOYING = "Deploying"
    BASELINE_UPGRADE = "BaselineUpgrade"
    UPGRADING = "Upgrading"
    UPGRADE_FAILED = "UpgradeFailed"
    READY = "Ready"

class ClusterUpgradeCadence(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Indicates when new cluster runtime version upgrades will be applied after they are released. By
    default is Wave0.
    """

    WAVE0 = "Wave0"  #: Cluster upgrade starts immediately after a new version is rolled out. Recommended for Test/Dev clusters.
    WAVE1 = "Wave1"  #: Cluster upgrade starts 7 days after a new version is rolled out. Recommended for Pre-prod clusters.
    WAVE2 = "Wave2"  #: Cluster upgrade starts 14 days after a new version is rolled out. Recommended for Production clusters.

class Direction(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Network security rule direction.
    """

    INBOUND = "inbound"
    OUTBOUND = "outbound"

class FailureAction(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The compensating action to perform when a Monitored upgrade encounters monitoring policy or
    health policy violations. Invalid indicates the failure action is invalid. Rollback specifies
    that the upgrade will start rolling back automatically. Manual indicates that the upgrade will
    switch to UnmonitoredManual upgrade mode.
    """

    ROLLBACK = "Rollback"  #: Indicates that a rollback of the upgrade will be performed by Service Fabric if the upgrade fails.
    MANUAL = "Manual"  #: Indicates that a manual repair will need to be performed by the administrator if the upgrade fails. Service Fabric will not proceed to the next upgrade domain automatically.

class ManagedClusterAddOnFeature(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Available cluster add-on features
    """

    DNS_SERVICE = "DnsService"
    BACKUP_RESTORE_SERVICE = "BackupRestoreService"
    RESOURCE_MONITOR_SERVICE = "ResourceMonitorService"

class ManagedIdentityType(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The type of managed identity for the resource.
    """

    NONE = "None"  #: Indicates that no identity is associated with the resource.
    SYSTEM_ASSIGNED = "SystemAssigned"  #: Indicates that system assigned identity is associated with the resource.
    USER_ASSIGNED = "UserAssigned"  #: Indicates that user assigned identity is associated with the resource.
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned, UserAssigned"  #: Indicates that both system assigned and user assigned identity are associated with the resource.

class ManagedResourceProvisioningState(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The provisioning state of the managed resource.
    """

    NONE = "None"
    CREATING = "Creating"
    CREATED = "Created"
    UPDATING = "Updating"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"
    DELETING = "Deleting"
    DELETED = "Deleted"
    OTHER = "Other"

class MoveCost(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Specifies the move cost for the service.
    """

    ZERO = "Zero"  #: Zero move cost. This value is zero.
    LOW = "Low"  #: Specifies the move cost of the service as Low. The value is 1.
    MEDIUM = "Medium"  #: Specifies the move cost of the service as Medium. The value is 2.
    HIGH = "High"  #: Specifies the move cost of the service as High. The value is 3.

class NsgProtocol(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Network protocol this rule applies to.
    """

    HTTP = "http"
    HTTPS = "https"
    TCP = "tcp"
    UDP = "udp"
    ICMP = "icmp"
    AH = "ah"
    ESP = "esp"

class PartitionScheme(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Enumerates the ways that a service can be partitioned.
    """

    SINGLETON = "Singleton"  #: Indicates that the partition is based on string names, and is a SingletonPartitionScheme object, The value is 0.
    UNIFORM_INT64_RANGE = "UniformInt64Range"  #: Indicates that the partition is based on Int64 key ranges, and is a UniformInt64RangePartitionScheme object. The value is 1.
    NAMED = "Named"  #: Indicates that the partition is based on string names, and is a NamedPartitionScheme object. The value is 2.

class ProbeProtocol(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """the reference to the load balancer probe used by the load balancing rule.
    """

    TCP = "tcp"
    HTTP = "http"
    HTTPS = "https"

class Protocol(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The reference to the transport protocol used by the load balancing rule.
    """

    TCP = "tcp"
    UDP = "udp"

class RollingUpgradeMode(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The mode used to monitor health during a rolling upgrade. The values are Monitored, and
    UnmonitoredAuto.
    """

    MONITORED = "Monitored"  #: The upgrade will stop after completing each upgrade domain and automatically monitor health before proceeding. The value is 0.
    UNMONITORED_AUTO = "UnmonitoredAuto"  #: The upgrade will proceed automatically without performing any health monitoring. The value is 1.

class ServiceCorrelationScheme(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The service correlation scheme.
    """

    ALIGNED_AFFINITY = "AlignedAffinity"  #: Aligned affinity ensures that the primaries of the partitions of the affinitized services are collocated on the same nodes. This is the default and is the same as selecting the Affinity scheme. The value is 0.
    NON_ALIGNED_AFFINITY = "NonAlignedAffinity"  #: Non-Aligned affinity guarantees that all replicas of each service will be placed on the same nodes. Unlike Aligned Affinity, this does not guarantee that replicas of particular role will be collocated. The value is 1.

class ServiceKind(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The kind of service (Stateless or Stateful).
    """

    STATELESS = "Stateless"  #: Does not use Service Fabric to make its state highly available or reliable. The value is 0.
    STATEFUL = "Stateful"  #: Uses Service Fabric to make its state or part of its state highly available and reliable. The value is 1.

class ServiceLoadMetricWeight(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Determines the metric weight relative to the other metrics that are configured for this
    service. During runtime, if two metrics end up in conflict, the Cluster Resource Manager
    prefers the metric with the higher weight.
    """

    ZERO = "Zero"  #: Disables resource balancing for this metric. This value is zero.
    LOW = "Low"  #: Specifies the metric weight of the service load as Low. The value is 1.
    MEDIUM = "Medium"  #: Specifies the metric weight of the service load as Medium. The value is 2.
    HIGH = "High"  #: Specifies the metric weight of the service load as High. The value is 3.

class ServicePackageActivationMode(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The activation Mode of the service package
    """

    SHARED_PROCESS = "SharedProcess"  #: Indicates the application package activation mode will use shared process.
    EXCLUSIVE_PROCESS = "ExclusiveProcess"  #: Indicates the application package activation mode will use exclusive process.

class ServicePlacementPolicyType(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The type of placement policy for a service fabric service. Following are the possible values.
    """

    INVALID_DOMAIN = "InvalidDomain"  #: Indicates that the ServicePlacementPolicyDescription is of type ServicePlacementInvalidDomainPolicyDescription, which indicates that a particular fault or upgrade domain cannot be used for placement of this service. The value is 0.
    REQUIRED_DOMAIN = "RequiredDomain"  #: Indicates that the ServicePlacementPolicyDescription is of type ServicePlacementRequireDomainDistributionPolicyDescription indicating that the replicas of the service must be placed in a specific domain. The value is 1.
    PREFERRED_PRIMARY_DOMAIN = "PreferredPrimaryDomain"  #: Indicates that the ServicePlacementPolicyDescription is of type ServicePlacementPreferPrimaryDomainPolicyDescription, which indicates that if possible the Primary replica for the partitions of the service should be located in a particular domain as an optimization. The value is 2.
    REQUIRED_DOMAIN_DISTRIBUTION = "RequiredDomainDistribution"  #: Indicates that the ServicePlacementPolicyDescription is of type ServicePlacementRequireDomainDistributionPolicyDescription, indicating that the system will disallow placement of any two replicas from the same partition in the same domain at any time. The value is 3.
    NON_PARTIALLY_PLACE_SERVICE = "NonPartiallyPlaceService"  #: Indicates that the ServicePlacementPolicyDescription is of type ServicePlacementNonPartiallyPlaceServicePolicyDescription, which indicates that if possible all replicas of a particular partition of the service should be placed atomically. The value is 4.

class ServiceScalingMechanismKind(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Enumerates the ways that a service can be partitioned.
    """

    SCALE_PARTITION_INSTANCE_COUNT = "ScalePartitionInstanceCount"  #: Represents a scaling mechanism for adding or removing instances of stateless service partition. The value is 0.
    ADD_REMOVE_INCREMENTAL_NAMED_PARTITION = "AddRemoveIncrementalNamedPartition"  #: Represents a scaling mechanism for adding or removing named partitions of a stateless service. The value is 1.

class ServiceScalingTriggerKind(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Enumerates the ways that a service can be partitioned.
    """

    AVERAGE_PARTITION_LOAD = "AveragePartitionLoad"  #: Represents a scaling trigger related to an average load of a metric/resource of a partition. The value is 0.
    AVERAGE_SERVICE_LOAD = "AverageServiceLoad"  #: Represents a scaling policy related to an average load of a metric/resource of a service. The value is 1.

class SkuName(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Sku Name. Basic requires a minimum of 3 nodes and Standard a minimum of 5. Basic only allows 1
    node type.
    """

    BASIC = "Basic"
    STANDARD = "Standard"
